@@ -30,35 +30,37 @@ pub enum RedisConfig {
 impl RedisConfig {
     pub fn addr(&self) -> impl Iterator<Item = &ConnectionAddr> {
         let boxed: Box<dyn Iterator<Item = &ConnectionAddr>> = match self {
-            RedisConfig::Single(conn) => Box::new(once(conn.addr.as_ref())),
-            RedisConfig::Cluster(conns) => Box::new(conns.iter().map(|conn| conn.addr.as_ref())),
+            RedisConfig::Single(conn) => Box::new(once(&conn.addr)),
+            RedisConfig::Cluster(conns) => Box::new(conns.iter().map(|conn| &conn.addr)),
         };
         boxed
     }
 
     pub fn db(&self) -> i64 {
         match self {
-            RedisConfig::Single(conn) => conn.db,
-            RedisConfig::Cluster(conns) => conns.first().map(|conn| conn.db).unwrap_or_default(),
+            RedisConfig::Single(conn) => conn.redis.db,
+            RedisConfig::Cluster(conns) => {
+                conns.first().map(|conn| conn.redis.db).unwrap_or_default()
+            }
         }
     }
 
     pub fn username(&self) -> Option<&str> {
         match self {
-            RedisConfig::Single(conn) => conn.username.as_deref(),
+            RedisConfig::Single(conn) => conn.redis.username.as_deref(),
             RedisConfig::Cluster(conns) => conns
                 .first()
-                .map(|conn| conn.username.as_deref())
+                .map(|conn| conn.redis.username.as_deref())
                 .unwrap_or_default(),
         }
     }
 
     pub fn passwd(&self) -> Option<&str> {
         match self {
-            RedisConfig::Single(conn) => conn.passwd.as_deref(),
+            RedisConfig::Single(conn) => conn.redis.password.as_deref(),
             RedisConfig::Cluster(conns) => conns
                 .first()
-                .map(|conn| conn.passwd.as_deref())
+                .map(|conn| conn.redis.password.as_deref())
                 .unwrap_or_default(),
         }
     }
@@ -125,6 +127,18 @@ pub enum NotAConfigError {
     NotAnArray(PathBuf),
 }
 
+#[derive(Debug)]
+pub enum SslOptions {
+    Enabled {
+        key: String,
+        cert: String,
+        ca: String,
+        verify: bool,
+    },
+    Disabled,
+    Default,
+}
+
 #[derive(Debug)]
 pub enum Database {
     Sqlite {
@@ -135,14 +149,14 @@ pub enum Database {
         username: String,
         password: String,
         connect: DbConnect,
-        disable_ssl: bool,
+        ssl_options: SslOptions,
     },
     Postgres {
         database: String,
         username: String,
         password: String,
         connect: DbConnect,
-        disable_ssl: bool,
+        ssl_options: SslOptions,
     },
 }
 
@@ -170,14 +184,25 @@ impl From<Database> for sqlx::any::AnyConnectOptions {
                 username,
                 password,
                 connect,
-                disable_ssl,
+                ssl_options,
             } => {
                 let mut options = MySqlConnectOptions::default()
                     .database(&database)
                     .username(&username)
                     .password(&password);
-                if disable_ssl {
-                    options = options.ssl_mode(MySqlSslMode::Disabled);
+                match ssl_options {
+                    SslOptions::Enabled { ca, verify, .. } => {
+                        options = options.ssl_ca(ca);
+                        options = options.ssl_mode(if verify {
+                            MySqlSslMode::VerifyIdentity
+                        } else {
+                            MySqlSslMode::VerifyCa
+                        });
+                    }
+                    SslOptions::Disabled => {
+                        options = options.ssl_mode(MySqlSslMode::Disabled);
+                    }
+                    SslOptions::Default => {}
                 }
                 match connect {
                     DbConnect::Socket(socket) => {
@@ -194,13 +219,13 @@ impl From<Database> for sqlx::any::AnyConnectOptions {
                 username,
                 password,
                 connect,
-                disable_ssl,
+                ssl_options,
             } => {
                 let mut options = PgConnectOptions::default()
                     .database(&database)
                     .username(&username)
                     .password(&password);
-                if disable_ssl {
+                if matches!(ssl_options, SslOptions::Disabled) {
                     options = options.ssl_mode(PgSslMode::Disable);
                 }
                 match connect {
